require_dependency "colorize"
require_dependency "token_auth_box/application_controller"

module TokenAuthBox
  class AuthController < ApplicationController
    include TokenAuthBox::Middleware

    def register
      user = User.new(user_params)
      if user.save
        data = user_response_data(user)
        token = TokenService.gen_token data

        render json: { user: data, token: token }
      else
        print_error user.errors.messages
        render json: user.errors.messages, status: :bad_request
      end
    end

    def login
      data = user_params
      user = User.find_by(email: data["email"])
        .try(:authenticate, data["password"])

      if !user
        print_error "No User Found; Check to see if the email/password combination is correct"
        render json: 'no', status: :unauthorized
      else 
        user_data = user_response_data(user)
        token = TokenService.gen_token user_data
        render json: { user: user_data, token: token }
      end
    end

    def verify
      begin
        user_data = TokenService.decode extract_token
        render json: user_data.to_json
      rescue StandardError => e
        print_error(e.message)
        render json: "Invalid Token: #{e.message}", status: :unauthorized
      end
    end


    private

    def extract_token
      auth_header = request.headers["authorization"]
      if auth_header
        request.headers["authorization"].split(" ")[1]
      else
        nil
      end
    end

    def user_params
      params.permit(:email, :password)
    end

    def user_response_data(user)
      attrs = user.attributes
      attrs.delete "password_digest"
      attrs
    end

    def print_error(msg)
      puts params
      puts "Authorization Heaader: #{request.headers["authorization"]}".yellow
      puts "Request Body: #{request.body.read}".yellow
      puts "Errors: #{msg.inspect}".red
    end
  end
end
